@@ -21,6 +21,8 @@ import sys
 import pkg_resources
 
 from astroid import builder
+from astroid import MANAGER
+from astroid.bases import  BUILTINS
 
 
 DATA_DIR = 'testdata/python{}/'.format(sys.version_info[0])
@@ -31,7 +33,7 @@ def find(name):
 
 
 def build_file(path, modname=None):
-     return builder.AstroidBuilder().file_build(find(path), modname)
+    return builder.AstroidBuilder().file_build(find(path), modname)
 
 
 class SysPathSetup(object):
@@ -46,3 +48,25 @@ class SysPathSetup(object):
                 del sys.path_importer_cache[key]
 
 
+class AstroidCacheSetupMixin(object):
+    """Mixin for handling the astroid cache problems.
+
+    When clearing the astroid cache, some tests fails due to
+    cache inconsistencies, where some objects had a different
+    builtins object referenced.
+    This saves the builtins module and makes sure to add it
+    back to the astroid_cache after the tests finishes.
+    The builtins module is special, since some of the
+    transforms for a couple of its objects (str, bytes etc)
+    are executed only once, so astroid_bootstrapping will be
+    useless for retrieving the original builtins module.
+    """
+
+    @classmethod
+    def setUpClass(cls):
+        cls._builtins = MANAGER.astroid_cache.get(BUILTINS)
+
+    @classmethod
+    def tearDownClass(cls):
+        if cls._builtins:
+            MANAGER.astroid_cache[BUILTINS] = cls._builtins
